/*
 * getut - get utmp values for a tty -
 * Copyright (C) August, 1999 Sergei Viznyuk <sv@phystech.com>
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <utmp.h>
#include <unistd.h>
#include <errno.h>

int main(argn,argc)
int argn;
char *argc[];
{
  char tname[128],*t;
  struct utmp ut,*ul;
  int s			=	1;
  int k			=	1;
  int i			=	1;
  unsigned p            =       0;
  
  while ( argc[i] )
    if ( argc[i][0]=='-' )
prgs: switch ( argc[i][s] )
	{
	  case 0:
	    i++;
	    s=1;
	    break;
	  case 'A':
	    s++;
	    p=0xffffffff;
	    goto prgs;
	  case 'd':
	    s++;
	    p|=32;
	    goto prgs;
	  case 'p':
	    s++;
	    p|=2;
	    goto prgs;
	  case 'u':
	    s++;
	    p|=16;
	    goto prgs;
	  case 'i':
	    s++;
	    p|=64;
	    goto prgs;
	  case 'c':
	    s++;
	    p|=1;
	    goto prgs;
	  case 'y':
	    s++;
	    p|=4;
	    goto prgs;
	  case 't':
	    s++;
	    p|=256;
	    goto prgs;
	  case 'b':
	    s++;
	    p|=8;
	    goto prgs;
	  case 's':
	    s++;
	    p|=128;
	    goto prgs;
          default:
usage:	    fprintf(stderr,"Usage: getut [-Abcdipstuy] [tty]\n");
	    exit(1);
	}
    else
      argc[k++]=argc[i++];
  if ( k > 1 )
    {
       struct stat sbuf;
       if ( argc[1][0] == '/' )
	 strncpy(tname,argc[1],23);
       else
	 {
	   strncpy(tname,"/dev/",5);
	   strncpy(&tname[5],argc[1],18);
	 }
       tname[23]=0;
       if ( lstat(tname,&sbuf) == -1 )
	 {
	   fprintf(stderr,"getut: %s does not exist\n",tname);
	   goto usage;
	 }
    }
  else
    {
      t=ttyname(STDIN_FILENO);
      if ( t )
        strncpy(tname,t,23);
      else
	{
	  fprintf(stderr,"getut: no tty attached to stdin\n");
	  goto usage;
	}
    }
  setutent();
  t=tname+5;
  strncpy(ut.ut_line,t,18);
  ul=getutline(&ut);
  if ( ! ul )
    {
       fprintf(stderr,"getut: %s\n",sys_errlist[errno]);
       exit(1);
    }
  ul->ut_user[UT_NAMESIZE-1]=0;
  if ( ! p ) p=0xffffffff;
  if ( p&1 )
    switch ( ul->ut_type )
     {
      case USER_PROCESS:
	fprintf(stdout,"user");
	break;
      case DEAD_PROCESS:
	fprintf(stdout,"dead");
	break;
      case INIT_PROCESS:
	fprintf(stdout,"init");
	break;
      case LOGIN_PROCESS:
	fprintf(stdout,"login");
	break;
      case RUN_LVL:
	fprintf(stdout,"runlevel");
	break;
      case BOOT_TIME:
	fprintf(stdout,"boot");
	break;
      case NEW_TIME:
	fprintf(stdout,"new");
	break;
      case OLD_TIME:
	fprintf(stdout,"old");
	break;
      case ACCOUNTING:
	fprintf(stdout,"accounting");
	break;
      default:
        fprintf(stdout,"unknown");
     }
  if ( p&2 )
    if ( p&1 )
      fprintf(stdout," %u",ul->ut_pid);
    else
      fprintf(stdout,"%u",ul->ut_pid);
  if ( p&4 )
    if ( p&3 )
      fprintf(stdout," %s",ul->ut_line);
    else
      fprintf(stdout,"%s",ul->ut_line);
  if ( p&8 )
    if ( p&7 )
      fprintf(stdout," %s",ul->ut_id);
    else
      fprintf(stdout,"%s",ul->ut_id);
  if ( p&16 )
    if ( p&15 )
      fprintf(stdout," %s",ul->ut_user);
    else
      fprintf(stdout,"%s",ul->ut_user);
  if ( p&32 )
    {
      i=strlen(ul->ut_host);
      t=ul->ut_host+i-1;
      while ( t > ul->ut_host )
	if ( *t-- == ':' )
	  i=0;
      if ( i )
	if ( p&31 )
	  fprintf(stdout," %s:0.0",ul->ut_host);
        else
	  fprintf(stdout,"%s:0.0",ul->ut_host);
      else
	if ( p&31 )
	  fprintf(stdout," %s",ul->ut_host);
        else
	  fprintf(stdout,"%s",ul->ut_host);
    }
  if ( p&64 )
    if ( p&63 )
      fprintf(stdout," %u.%u.%u.%u",
	  (((unsigned char *)&ul->ut_addr)[0]),
	  (((unsigned char *)&ul->ut_addr)[1]),
	  (((unsigned char *)&ul->ut_addr)[2]),
	  (((unsigned char *)&ul->ut_addr)[3]));
    else
      fprintf(stdout,"%u.%u.%u.%u",
	  ((unsigned char *)&ul->ut_addr)[0],
	  ((unsigned char *)&ul->ut_addr)[1],
	  ((unsigned char *)&ul->ut_addr)[2],
	  ((unsigned char *)&ul->ut_addr)[3]);
  if ( p&128 )
    if ( p&127 )
      fprintf(stdout," %lu",ul->ut_session);
    else
      fprintf(stdout,"%lu",ul->ut_session);
  if ( p&256 )
     {
       time_t timep;
       struct tm *mtm;
       timep=ul->ut_tv.tv_sec;
       mtm=localtime(&timep);
       if ( p&255 )
         fprintf(stdout," %02d:%02d:%02d",
	 mtm->tm_hour,mtm->tm_min,mtm->tm_sec);
       else
         fprintf(stdout,"%02d:%02d:%02d",
	 mtm->tm_hour,mtm->tm_min,mtm->tm_sec);
     }
  fprintf(stdout,"\n");
  endutent();
  exit(0);
}
